@@ -1,7 +1,7 @@
 #
 # This file is part of Config-Model-TkUI
 #
-# This software is Copyright (c) 2014 by Dominique Dumont.
+# This software is Copyright (c) 2015 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
@@ -26,7 +26,7 @@
 #    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 
 package Config::Model::Tk::AnyViewer;
-$Config::Model::Tk::AnyViewer::VERSION = '1.344';
+$Config::Model::Tk::AnyViewer::VERSION = '1.349';
 use strict;
 use warnings;
 use Carp;
@@ -72,7 +72,7 @@ sub add_header {
     $cw->{config_class_name} = $class;
 
     my $label = "$type: ";
-    $label .= $item->location || "Class $class";
+    $label .= $item->location_short || "Class $class";
     my $f = $cw->Frame;
 
     $f->Label( -image => $img{ lc($type) }, -anchor => 'w' )->pack( -side => 'left' );
@@ -125,7 +125,8 @@ sub add_help {
 
     #    if (  $force_text_widget eq 'pod' or $help =~ /\n=\w+|[A-Z]</ ) {
     if ( $force_text_widget or $help =~ /\n/ or length($help) > 50 ) {
-        $widget = $help_frame->PodText( -height => 5, -scrollbars => 'oe' );
+        $widget = $help_frame->PodText( -height => 6, -scrollbars => 'oe' );
+        $widget->base_font_size(12);
         $widget->pack(@fxe1);
         $cw->update_help( $widget, $help );
     }
