@@ -1,17 +1,14 @@
 #
 # This file is part of Config-Model-TkUI
 #
-# This software is Copyright (c) 2013 by Dominique Dumont.
+# This software is Copyright (c) 2015 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::Tk::HashEditor;
-{
-  $Config::Model::Tk::HashEditor::VERSION = '1.340';
-}
-
+$Config::Model::Tk::HashEditor::VERSION = '1.349';
 use strict;
 use warnings;
 use Carp;
@@ -34,10 +31,7 @@ my $logger = Log::Log4perl::get_logger("Tk::HashEditor");
 
 my $entry_width = 15;
 
-my (
-    $up_img,     $down_img,   $add_img, $rm_img,
-    $eraser_img, $remove_img, $rename_img, $copy_img
-);
+my ( $up_img, $down_img, $add_img, $rm_img, $eraser_img, $remove_img, $rename_img, $copy_img );
 *icon_path = *Config::Model::TkUI::icon_path;
 
 sub ClassInit {
@@ -64,10 +58,9 @@ my %widget_activation_table = (
 sub Populate {
     my ( $cw, $args ) = @_;
     my $hash = $cw->{hash} = delete $args->{-item}
-      || die "HashEditor: no -item, got ", keys %$args;
+        || die "HashEditor: no -item, got ", keys %$args;
     delete $args->{-path};
-    $cw->{store_cb} = delete $args->{-store_cb} || die __PACKAGE__,
-      "no -store_cb";
+    $cw->{store_cb} = delete $args->{-store_cb} || die __PACKAGE__, "no -store_cb";
 
     unless ( defined $up_img ) {
         $add_img    = $cw->Photo( -file => $icon_path . 'add.png' );
@@ -84,7 +77,7 @@ sub Populate {
     my $inst = $hash->instance;
 
     # frame for element list
-    my $elt_frame = $cw->Frame(qw/-relief raised -borderwidth 2/) ->pack( @fbe1 );
+    my $elt_frame = $cw->Frame(qw/-relief raised -borderwidth 2/)->pack(@fbe1);
 
     $elt_frame->Label( -text => $hash->element_name . ' elements' )->pack(@fx);
 
@@ -96,40 +89,46 @@ sub Populate {
         -height     => 6,
     );
     $tklist->pack( @fbe1, -side => 'left' );
-    $tklist->insert( end => $hash->fetch_all_indexes );
     $cw->Advertise( tklist => $tklist );
+    $cw->insert( end => $hash->fetch_all_indexes );
 
-    my $item_frame =
-      $cw->Frame(qw/-borderwidth 1 -relief groove/)
-      ->pack( @fx, -anchor => 'n' );
+    my $item_frame = $cw->Frame(qw/-borderwidth 1 -relief groove/)->pack( @fx, -anchor => 'n' );
 
     my $balloon = $cw->Balloon( -state => 'balloon' );
 
     my $label_keep_frame = $item_frame->Frame->pack(@fxe1);
     my $item             = '';
     my $keep             = 0;
-    $label_keep_frame->Label( -text => 'Item:' )
-      ->pack( -side => 'left', -anchor => 'w' );
+    $label_keep_frame->Label( -text => 'Item:' )->pack( -side => 'left', -anchor => 'w' );
 
-    # copy selected entry text into item (textvariable) when $keep is set
-    my $keep_cb = sub {
+    # clear entry
+    my $clear_b = $label_keep_frame->Button(
+        -command  => sub { $item = ''; },
+        -text     => 'clear'
+    )->pack(qw/-side right -anchor e/);
+    $balloon->attach( $clear_b, -msg => 'clear entry below' );
+
+    # copy select entry
+    my $copy_cb = sub {
         my $sel = $tklist->curselection;
-        $item = $keep && $sel ? $tklist->get($sel) : '';
+        $item = $tklist->get($sel) if $sel;
     };
+    my $copy_b = $label_keep_frame->Button(
+        -command  => $copy_cb,
+        -text     => 'copy'
+    )->pack(qw/-side right -anchor e/);
+    $balloon->attach( $copy_b, -msg => 'copy selected item in entry below' );
 
     my $keep_b = $label_keep_frame->Checkbutton(
         -variable => \$keep,
-        -command  => $keep_cb,
         -text     => 'keep'
     )->pack(qw/-side right -anchor e/);
-    $balloon->attach( $keep_b,
-        -msg => 'keep entry in widget after add, move or copy' );
+    $balloon->attach( $keep_b, -msg => 'keep content of entry below after add, move or copy' );
 
     # Entry
     my $entry = $item_frame->Entry( -textvariable => \$item );
     $entry->pack( @fxe1, qw/-side top -anchor n/ );
-    $balloon->attach( $entry,
-        -msg => 'enter item name to add, copy to, or move to' );
+    $balloon->attach( $entry, -msg => 'enter item name to add, copy to, or move to' );
     $cw->Advertise( entry => $entry );
 
     # bind both entries to update correctly the state of all buttons
@@ -139,7 +138,7 @@ sub Populate {
             tklist => $tklist->curselection || 0
         );
     };
-    $entry->bind( '<KeyPress>', $bound_sub );
+    $entry->bind( '<KeyPress>',         $bound_sub );
     $entry->bind( '<B2-ButtonRelease>', $bound_sub );
     $tklist->bind( '<<ListboxSelect>>', $bound_sub );
 
@@ -158,8 +157,7 @@ sub Populate {
     $addb->pack( @fxe1, qw/-side left/ );
     $cw->Advertise( add => $addb );
     my $add_str = $hash->ordered ? " after selection" : '';
-    $balloon->attach( $addb,
-        -msg => "fill field above and click to add new entry" . $add_str );
+    $balloon->attach( $addb, -msg => "fill field above and click to add new entry" . $add_str );
 
     # copy button
     my $cp_b = $button_frame->Button(
@@ -170,7 +168,7 @@ sub Populate {
             &$bound_sub;
         },
     );
-    $cp_b->pack(@fxe1,qw/-side right/);
+    $cp_b->pack( @fxe1, qw/-side right/ );
     $cw->Advertise( @fxe1, cp => $cp_b );
     $balloon->attach( $cp_b, -msg => "copy selected item in entry" );
 
@@ -188,7 +186,7 @@ sub Populate {
     $balloon->attach( $rename_b, -msg => "rename selected key in entry" );
 
     if ( $hash->ordered ) {
-         my $up_b = $button_frame->Button(
+        my $up_b = $button_frame->Button(
             -image   => $up_img,
             -command => sub { $cw->move_selected_up; },
         );
@@ -235,6 +233,24 @@ sub Populate {
     $cw->Tk::Frame::Populate($args);
 }
 
+# the following function is used to make multi-line keys (like Files
+# entries in Debian copyright files) more manageable: LF are replaced
+# by \n to enable editing a multi line entry in a Tk::Entry.
+sub insert {
+    my $cw = shift ;
+    my $where = shift ;
+    my @what = map { s/\n/\\n/g; $_; }
+        #map { $hash->shorten_idx($_); }
+        @_ ;
+    $cw->Subwidget('tklist')->insert($where => @what);
+}
+
+# this function (not a method) restore the LF in a multi line key
+# (reverse the operation done above
+sub restore_keys {
+    return map { s/\\n/\n/g; $_; } @_ ;
+}
+
 sub remove_all_elements {
     my $cw     = shift;
     my $dialog = $cw->Dialog(
@@ -275,7 +291,7 @@ sub add_entry {
 
     $logger->debug("add_entry: $add");
 
-    if ( $hash->exists($add) ) {
+    if ( $hash->exists(restore_keys($add)) ) {
         $cw->Dialog(
             -title => "Add item error",
             -text  => "Entry $add already exists",
@@ -284,7 +300,7 @@ sub add_entry {
     }
 
     # add entry in hash
-    eval { $hash->fetch_with_id($add) };
+    eval { $hash->fetch_with_id(restore_keys($add)) };
 
     if ($@) {
         $cw->Dialog(
@@ -304,18 +320,17 @@ sub add_entry {
     if ( @selected and $hash->ordered ) {
         my $idx = $tklist->get( $selected[0] );
         $logger->debug("add_entry on ordered hash: swap $idx and $add");
-        $hash->move_after( $add, $idx );
-        $logger->debug(
-            "new hash idx: " . join( ',', $hash->get_all_indexes ) );
+        $hash->move_after( restore_keys($add, $idx) );
+        $logger->debug( "new hash idx: " . join( ',', $hash->fetch_all_indexes ) );
         my $new_idx = $selected[0] + 1;
-        $tklist->insert( $new_idx, $add );
+        $cw->insert( $new_idx, $add );
         $tklist->selectionSet($new_idx);
         $tklist->see($new_idx);
     }
     elsif ( $hash->ordered ) {
 
         # without selection on ordered hash, items are simply pushed
-        $tklist->insert( 'end', $add );
+        $cw->insert( 'end', $add );
         $tklist->selectionSet('end');
         $tklist->see('end');
     }
@@ -337,9 +352,9 @@ sub add_and_sort_item {
     my $added  = 0;
 
     $tklist->selectionClear( 0, 'end' );
-    foreach ( $tklist->get( 0, 'end' ) ) {
-        if ( $add lt $_ ) {
-            $tklist->insert( $idx, $add );
+    foreach my $item ( $tklist->get( 0, 'end' ) ) {
+        if ( $add lt $item ) {
+            $cw->insert( $idx, $add );
             $tklist->selectionSet($idx);
             $tklist->see($idx);
             $added = 1;
@@ -349,7 +364,7 @@ sub add_and_sort_item {
     }
 
     if ( not $added ) {
-        $tklist->insert( 'end', $add );    # last entry
+        $cw->insert( 'end', $add );    # last entry
         $tklist->selectionSet('end');
         $tklist->see('end');
     }
@@ -366,7 +381,7 @@ sub add_item {
     if ( $hash->ordered ) {
         $logger->debug("add_item: adding $add in ordered hash");
         $tklist->selectionClear( 0, 'end' );
-        $tklist->insert( 'end', $add );
+        $cw->insert( 'end', $add );
         $tklist->selectionSet('end');
         $tklist->see('end');
     }
@@ -409,11 +424,9 @@ sub copy_selected_in {
 
     my $hash = $cw->{hash};
 
-    my $new_idx = $hash->exists($to_name) ? 0 : 1;
-    $logger->debug(
-        "copy_selected_to: from $from_name to $to_name (is new index: $new_idx)"
-    );
-    $hash->copy( $from_name, $to_name );
+    my $new_idx = $hash->exists(restore_keys($to_name)) ? 0 : 1;
+    $logger->debug( "copy_selected_to: from $from_name to $to_name (is new index: $new_idx)" );
+    $hash->copy( restore_keys($from_name, $to_name) );
 
     if ($new_idx) {
         $logger->debug("copy_selected_to: add_item $to_name");
@@ -442,13 +455,13 @@ sub move_selected_to {
     my $hash = $cw->{hash};
     $tklist->delete(@from_idx);
 
-    my $new_idx = $hash->exists($to_name) ? 0 : 1;
-    $hash->move( $from_name, $to_name );
+    my $new_idx = $hash->exists(restore_keys($to_name)) ? 0 : 1;
+    $hash->move( restore_keys($from_name, $to_name) );
 
     if ($new_idx) {
         if ( $hash->ordered ) {
             $tklist->selectionClear( 0, 'end' );
-            $tklist->insert( $from_idx[0], $to_name );
+            $cw->insert( $from_idx[0], $to_name );
             $tklist->selectionSet( $from_idx[0] );
         }
         else {
@@ -473,14 +486,13 @@ sub move_selected_up {
     $logger->debug("move_selected_up: $name (@idx)");
     $tklist->delete(@idx);
     my $new_idx = $idx[0] - 1;
-    $tklist->insert( $new_idx, $name );
+    $cw->insert( $new_idx, $name );
     $tklist->selectionSet($new_idx);
     $tklist->see($new_idx);
 
     my $hash = $cw->{hash};
-    $hash->move_up($name);
-    $logger->debug(
-        "move_up new hash idx: " . join( ',', $hash->get_all_indexes ) );
+    $hash->move_up(restore_keys($name));
+    $logger->debug( "move_up new hash idx: " . join( ',', $hash->fetch_all_indexes ) );
 
     $cw->reload_tree;
 }
@@ -490,7 +502,7 @@ sub move_selected_down {
     my $tklist = $cw->Subwidget('tklist');
     my @idx    = $tklist->curselection();
     my $hash   = $cw->{hash};
-    my @h_idx  = $hash->get_all_indexes;
+    my @h_idx  = $hash->fetch_all_indexes;
 
     return unless @idx and $idx[0] < $#h_idx;
 
@@ -499,13 +511,12 @@ sub move_selected_down {
     $logger->debug("move_selected_down: $name (@idx)");
     $tklist->delete(@idx);
     my $new_idx = $idx[0] + 1;
-    $tklist->insert( $new_idx, $name );
+    $cw->insert( $new_idx, $name );
     $tklist->selectionSet($new_idx);
     $tklist->see($new_idx);
 
-    $hash->move_down($name);
-    $logger->debug(
-        "move_down new hash idx: " . join( ',', $hash->get_all_indexes ) );
+    $hash->move_down(restore_keys($name));
+    $logger->debug( "move_down new hash idx: " . join( ',', $hash->fetch_all_indexes ) );
 
     $cw->reload_tree;
 }
@@ -517,7 +528,7 @@ sub delete_selection {
 
     foreach ( $tklist->curselection() ) {
         my $idx = $tklist->get($_);
-        $hash->delete($idx);
+        $hash->delete(restore_keys($idx));
         $tklist->delete($_);
         $cw->reload_tree;
     }
